import java.util.*;
import java.io.*;

public class Align {
    
	// general information
    public static SpliceGraph s;
	
	public static Edge[] edge_list;
    public static String[] reads;
    public static ArrayList[] align;
    public static ArrayList[] confidence;
    public static double[] c;
	public static int fasta_readlen;

	public static ArrayList<Integer[]> align1;

	public static HashMap<String,ArrayList<Integer>> first_ids;
    
    public static void main(String[] args) {
		if (args.length != 3 && args.length != 4) {
			System.out.println("usage: Align <psg> <bowtie.align> <FASTA readlen> [<beta>]");
			System.exit(0);
		}
		
		s = new SpliceGraph(args[0]);
		
		String filename = args[1];
		fasta_readlen = Integer.parseInt(args[2]);
		process_reads(filename);
		
		try {
			s.setBeta(Double.parseDouble(args[3]));
		} catch (ArrayIndexOutOfBoundsException e) {
			// do nothing, this is okay
		}
		catch (NumberFormatException nfe) {
			System.out.print("Error: Beta should be given as a double.");
			System.exit(0);
		}
		
		// checking the edge case where we don't have any aligned reads
		if (reads.length == 0) {
			double ll = 0.0;
			//System.out.println("\tNumber of null reads: 0");
			System.out.println("\tNumber of reads: 0");
			s.emOutput(ll,-1);
			
			System.out.println();
			System.out.println("Current log likelihood: "+ll);
			System.out.println("Converged after 0 iterations");
			System.out.println("Log likelihood diff: 0");
			System.out.println("Max edge difference: 0.0");
			System.out.println("L1 norm: 0.0");
			System.out.println("L2 norm: 0.0");
			return;
		}
		
		// so if we're here, then we definitely have reads
		System.out.println("\tNumber of reads: "+reads.length);

		// int num_align = 0;
		// for (int i=0; i<align.length; i++) {
		// 	num_align += align[i].size();
		// }
		// System.out.println("\tNumber of alignments: "+num_align);
		
		double[] initial = s.getEdgeWeights();
		s.resetEdgeWeights();
		s.learnEM(reads,align,confidence,1);
		
		double[] a = s.getEdgeWeights();
		double result = 0;
		for (int i=0; i<initial.length; i++) {
			result += Math.abs(a[i]-initial[i]);
		}
		System.out.println("L1 norm: "+result);
		result = 0;
		for (int i=0; i<initial.length; i++) {
			result += Math.pow(a[i]-initial[i],2);
		}
		result = Math.sqrt(result);
		System.out.println("L2 norm: "+result);
    }
	
	// reads information from a bowtie alignment file
    private static void process_reads(String filename) {
		// initialize the temporary variables
		ArrayList<Integer> start = new ArrayList<Integer>();
		ArrayList<String> read = new ArrayList<String>();
		ArrayList<String> alig = new ArrayList<String>();
		ArrayList<String> which = new ArrayList<String>();
		ArrayList<String> mismatch = new ArrayList<String>();
		ArrayList<String> quality = new ArrayList<String>();
		ArrayList<Integer> length = new ArrayList<Integer>();
		
		// initialize the global variables
		reads = null;
		align = null;
		confidence = null;
		c = null;
		
		// retrieve records from alignment file
		ArrayList<AlignmentRecord> records = Utility.read_alignment_file(filename);
		for (int i=0; i<records.size(); i++) {
			AlignmentRecord r = records.get(i);
			which.add(r.id);
			alig.add(r.alignment);
			start.add(r.start_position);
			read.add(r.read);
			length.add(r.read_length);
			quality.add(r.quality);
			mismatch.add(r.mismatch);
		}
		
		// compute quality scores and find all unique ids
		c = Utility.compute_quality(quality,mismatch);
		first_ids = Utility.make_id_map(which);

		// create read list from list of unique ids
		Object[] unique_ids = first_ids.keySet().toArray();
		reads = new String[unique_ids.length];
		for (int i=0; i<unique_ids.length; i++) {
			int read_index = first_ids.get((String)unique_ids[i]).get(0).intValue();
			reads[i] = read.get(read_index);
		}
		
		// make the alignments useable
		if (reads.length > 0) {
			process_align(unique_ids,alig,start,length);
		}
    }

    // utility method to find the location of an Integer[] in an ArrayList
    private static ArrayList<Integer> indexOf(ArrayList<Integer[]> a, Integer[] b) {
		ArrayList<Integer> retval = new ArrayList<Integer>();
		for (int i=0; i<a.size(); i++) {
			if (a.get(i).length == b.length) {
				boolean verify = true;
				for (int j=0; j<a.get(i).length; j++) {
					if (!a.get(i)[j].equals(b[j])) {
						verify = false;
					}
				}
				if (verify) {
					retval.add(i);
				}
			}
		}
		return retval;
    }

    private static void trim(Object[] id, 
							 ArrayList<String> al,
							 ArrayList<Integer> st,
							 ArrayList<Integer> len) {
    	// Here, we translate String representations of alignments
    	// into Integer[] representations so we can use them
    	//
    	// The indices are the same as those stored in the first_ids HashMap
    	align1 = new ArrayList<Integer[]>(al.size());
    	for (int i=0; i<al.size(); i++) {
    		align1.add(new Integer[1]);
    	}

    	// Go through the unique read IDs and sort everything out:
    	for (int i=0; i<id.length; i++) {
    		String read_id = (String) id[i];							// get the read's id
    		ArrayList<Integer> read_indices = first_ids.get(read_id);	// get all instances of that id
    		for (int j=0; j<read_indices.size(); j++) {
   				int duplicate_index = read_indices.get(j).intValue();
   				String[] match = al.get(duplicate_index).substring(1).split(":");
   				Integer[] path = new Integer[match.length];
   				for (int k=0; k<match.length; k++) {
   					path[k] = Integer.decode(match[k]);					// create Integer[] representation
   				}
   				Integer[] t = null;
   				t = Utility.adjust_alignment(path,st.get(duplicate_index),len.get(duplicate_index),s,fasta_readlen);
   				align1.set(duplicate_index,t);

				int k=0;												// determine whether start index needs adjustment
				int new_st = st.get(duplicate_index);
				if (path[k].intValue() != t[0].intValue()) {			// remove one leading exon
					new_st -= Math.min(s.getSegment(path[k].intValue()).length(),fasta_readlen-1);
					k++;
				}
				else {
					if (path.length == 1) {
						// we're fine, this was a single exon alignment
					}
					else {
						// multiple exon alignment; add the rest of the first segment back in
						if (s.getSegment(path[k].intValue()).length()-(fasta_readlen-1) > 0) {
							new_st += s.getSegment(path[k].intValue()).length() - (fasta_readlen - 1);
						}
					}
				}
				while (path[k].intValue() != t[0].intValue() && new_st > 0) {
					// remove any other extraneous leading exons
					new_st -= s.getSegment(path[k].intValue()).length();
					k++;
				}
				st.set(duplicate_index,new_st);
			}
    	}
    }
	
	private static void process_align(Object[] id, 
									  ArrayList<String> al,
									  ArrayList<Integer> st,
									  ArrayList<Integer> len) {
		align = new ArrayList[reads.length];
		confidence = new ArrayList[reads.length];

		// first: translate alignments to Integer[] and trim to fit
		trim(id,al,st,len);

		// second: remove any duplicate alignments (side effect of fasta_readlen)
		ArrayList[] starts = new ArrayList[id.length];		
		for (int i=0; i<id.length; i++) {
			String read_id = (String) id[i];							// get the read's id
    		ArrayList<Integer> read_indices = first_ids.get(read_id);	// get all instances of that id
			align[i] = new ArrayList<Integer[]>();
			starts[i] = new ArrayList<Integer>();
			confidence[i] = new ArrayList<Double>();

			for (int j=0; j<read_indices.size(); j++) {
				int index = read_indices.get(j).intValue();				// the relevant index in the bowtie file
				if (!Utility.contains(align[i],align1.get(index))) {
					// no problems, add everything
					align[i].add(align1.get(index));
					starts[i].add(st.get(index));
					confidence[i].add(c[index]);
				}
				else {
					ArrayList<Integer> temp = indexOf(align[i],align1.get(index));
					boolean contained = false;
					for (int k=0; k<temp.size(); k++) {
						if (starts[i].get(temp.get(k).intValue()).equals(st.get(index))) {
							contained = true;
							break;
						}
					}
					if (!contained) {
						// different alignment to same path
						align[i].add(align1.get(index));
						starts[i].add(st.get(index));
						confidence[i].add(c[index]);
					}
				}
			}
		}
    }	
}
