import java.io.*;

public class First35 {
    public static void main(String[] args) {
	if (args.length != 3 && args.length != 2) {
	    System.out.println("Usage: First35 [-v] psg numTrial");
	    System.exit(0);
	}

	int verbose = 0;
	if (args.length == 3) { verbose = 1; }

	String filename = args[0];
	int numTrial = Integer.parseInt(args[verbose+1]);

	SpliceGraph s = new SpliceGraph(filename);

	double[] initial = s.getEdgeWeights();
	double read_1_diff = 0, read_1_m = 0;
	double read_2_diff = 0, read_2_m = 0;
	double read_3_diff = 0, read_3_m = 0;
	double read_4_diff = 0, read_4_m = 0;
	double read_5_diff = 0, read_5_m = 0;
	double read_6_diff = 0, read_6_m = 0;
	double read_7_diff = 0, read_7_m = 0;

	for (int i=0; i<numTrial; i++) {
	    s = new SpliceGraph(filename);
	    String[] reads_1 = trim(s.generateReads(10000,50));
	    String[] reads_2 = trim(s.generateReads(10000,75));
	    String[] reads_3 = trim(s.generateReads(10000,100));
	    String[] reads_4 = trim(s.generateReads(10000,125));
	    String[] reads_5 = trim(s.generateReads(10000,150));
	    String[] reads_6 = trim(s.generateReads(10000,175));
	    String[] reads_7 = trim(s.generateReads(10000,200));
	    s.resetEdgeWeights();
	    s.learnEM(reads_1,verbose);
	    read_1_diff += euclid(initial,s.getEdgeWeights());
	    read_1_m += manhattan(initial,s.getEdgeWeights());
	    //System.out.println("d="+manhattan(initial,s.getEdgeWeights()));
	    if (verbose == 0) { System.out.print("."); }
	    s.resetEdgeWeights();
	    s.learnEM(reads_2,verbose);
	    read_2_diff += euclid(initial,s.getEdgeWeights());
	    read_2_m += manhattan(initial,s.getEdgeWeights());
	    //System.out.println("d="+manhattan(initial,s.getEdgeWeights()));
	    if (verbose == 0) { System.out.print("."); }
	    s.resetEdgeWeights();
	    s.learnEM(reads_3,verbose);
	    read_3_diff += euclid(initial,s.getEdgeWeights());
	    read_3_m += manhattan(initial,s.getEdgeWeights());
	    //System.out.println("d="+manhattan(initial,s.getEdgeWeights()));
	    if (verbose == 0) { System.out.print("."); }
	    s.resetEdgeWeights();
	    s.learnEM(reads_4,verbose);
	    read_4_diff += euclid(initial,s.getEdgeWeights());
	    read_4_m += manhattan(initial,s.getEdgeWeights());
	    //System.out.println("d="+manhattan(initial,s.getEdgeWeights()));
	    if (verbose == 0) { System.out.print("."); }
	    s.resetEdgeWeights();
	    s.learnEM(reads_5,verbose);
	    read_5_diff += euclid(initial,s.getEdgeWeights());
	    read_5_m += manhattan(initial,s.getEdgeWeights());
	    //System.out.println("d="+manhattan(initial,s.getEdgeWeights()));
	    if (verbose == 0) { System.out.print("."); }
	    s.resetEdgeWeights();
	    s.learnEM(reads_6,verbose);
	    read_6_diff += euclid(initial,s.getEdgeWeights());
	    read_6_m += manhattan(initial,s.getEdgeWeights());
	    //System.out.println("d="+manhattan(initial,s.getEdgeWeights()));
	    if (verbose == 0) { System.out.print("."); }
	    s.resetEdgeWeights();
	    s.learnEM(reads_7,verbose);
	    read_7_diff += euclid(initial,s.getEdgeWeights());
	    read_7_m += manhattan(initial,s.getEdgeWeights());
	    if (verbose == 0) { System.out.print("."); }
	}
	System.out.println();

	read_1_diff /= numTrial;
	read_2_diff /= numTrial;
	read_3_diff /= numTrial;
	read_4_diff /= numTrial;
	read_5_diff /= numTrial;
	read_6_diff /= numTrial;
	read_7_diff /= numTrial;
	read_1_m /= numTrial;
	read_2_m /= numTrial;
	read_3_m /= numTrial;
	read_4_m /= numTrial;
	read_5_m /= numTrial;
	read_6_m /= numTrial;
	read_7_m /= numTrial;

	System.out.println("Results, averaged over "+numTrial+" runs:");
	System.out.println(" 50 bases: "+read_1_diff+", "+read_1_m);
	System.out.println(" 75 bases: "+read_2_diff+", "+read_2_m);
	System.out.println("100 bases: "+read_3_diff+", "+read_3_m);
	System.out.println("125 bases: "+read_4_diff+", "+read_4_m);
	System.out.println("150 bases: "+read_5_diff+", "+read_5_m);
	System.out.println("175 bases: "+read_6_diff+", "+read_6_m);
	System.out.println("200 bases: "+read_7_diff+", "+read_7_m);
    }

    // trims the read to the first 35 bases
    private static String[] trim(String[] reads) {
	String[] result = new String[reads.length];
	for (int i=0; i<reads.length; i++) {
	    result[i] = reads[i].substring(0,35);
	    //result[i] = reads[i].substring(reads[i].length()-35);
	}
	return result;
    }

    // calculates Euclidean distance between two vectors
    private static double euclid(double[] a, double[] b) {
	double result = 0;
	for (int i=0; i<a.length; i++) {
	    result += Math.pow(a[i]-b[i],2);
	}
	return Math.sqrt(result);
    }
    
    // calculates Manhattan distance between two vectors
    private static double manhattan(double[] a, double[] b) {
	double result = 0;
	for (int i=0; i<a.length; i++) {
	    result += Math.abs(a[i]-b[i]);
	}
	return result;
    }
}