import java.util.*;
import java.io.*;

public class Utility {
 
    public static String[] StringArray(ArrayList<String> lines) {
		String[] s = new String[lines.size()];
		for (int i=0; i<lines.size(); i++) {
		    s[i] = lines.get(i);
		}
		return s;
    }
	
	public static int[][] IntArray(ArrayList[] ints) {
		int[][] s = new int[ints.length][];
		for (int i=0; i<ints.length; i++) {
			s[i] = new int[ints[i].size()];
			for (int j=0; j<ints[i].size(); j++) {
				s[i][j] = ((Integer) ints[i].get(j)).intValue();
			}
		}
		return s;
	}

    public static int[] IntArray(ArrayList<Integer> ints) {
		int[] s = new int[ints.size()];
		for (int i=0; i<ints.size(); i++) {
		    s[i] = ints.get(i).intValue();
		}
		return s;
    }

    public static Integer[] IntegerArray(ArrayList<Integer> ints) {
		Integer[] s = new Integer[ints.size()];
		for (int i=0; i<ints.size(); i++) {
		    s[i] = ints.get(i);
		}
		return s;
    }

    public static Integer[] IntegerArray(LinkedList<Integer> ints) {
		Integer[] s = new Integer[ints.size()];
		for (int i=0; i<ints.size(); i++) {
		    s[i] = ints.get(i);
		}
		return s;
    }

    public static Segment[] SegArray(ArrayList<Segment> lines) {
		Segment[] s = new Segment[lines.size()];
		for (int i=0; i<lines.size(); i++) {
		    s[i] = lines.get(i);
		}
		return s;
    }

    public static Edge[] EdgArray(ArrayList<Edge> lines) {
		Edge[] s = new Edge[lines.size()];
		for (int i=0; i<lines.size(); i++) {
		    s[i] = lines.get(i);
		}
		return s;
    }

    public static boolean Acontains(ArrayList<Integer[]> a, Integer[] b) {
		for (int i=0; i<a.size(); i++) {
		    if (equals(a.get(i),b)) {
				return true;
		    }
		}
		return false;
    }

    public static boolean equals(Integer[] a, Integer[] b) {
		if (a.length != b.length) {
		    return false;
		}
		for (int i=0; i<a.length; i++) {
		    if (!a[i].equals(b[i])) {
			return false;
		    }
		}
		return true;
    }

    public static boolean contains(ArrayList<String> a, String b) {
		boolean retval = false;
		for (int i=0; i<a.size(); i++) {
			if (a.get(i).equals(b)) {
				retval = true;
			}
		}
		return retval;
    }
	
    public static boolean contains(ArrayList<Integer[]> a, Integer[] b) {
		boolean retval = false;
		for (int i=0; i<a.size(); i++) {
			if (a.get(i).length == b.length) {
				boolean verify = true;
				for (int j=0; j<a.get(i).length; j++) {
					if (!a.get(i)[j].equals(b[j])) {
						verify = false;
					}
				}
				if (verify) {
					retval = true;
					break;
				}
			}
		}
		return retval;
    }

    // create a HashMap with pointers to all instances of a read ID
    public static HashMap<String,ArrayList<Integer>> make_id_map (ArrayList<String> id_list) {
    	HashMap<String,ArrayList<Integer>> result = new HashMap<String,ArrayList<Integer>>();
    	for (int i=0; i<id_list.size(); i++) {
			if (result.containsKey(id_list.get(i))) {
				result.get(id_list.get(i)).add(i);
			}
			else {
				ArrayList<Integer> t = new ArrayList<Integer>();
				t.add(i);
				result.put(id_list.get(i),t);
			}
		}
    	return result;
    }

    // read an alignment file into an array of AlignmentRecords
    public static ArrayList<AlignmentRecord> read_alignment_file (String filename) {
    	ArrayList<AlignmentRecord> results = new ArrayList<AlignmentRecord>();
    	try {
    		Scanner r = new Scanner(new File(filename));
    		while (r.hasNextLine()) {
    			String[] components = r.nextLine().split("\t");
    			AlignmentRecord temp_record = new AlignmentRecord();
    			temp_record.id = components[0];
				temp_record.alignment = components[2];
				temp_record.start_position = Integer.decode(components[3]);
				temp_record.read = components[4];
				temp_record.read_length = components[4].length();
				temp_record.quality = components[5];
				if (components.length == 8) {
					temp_record.mismatch = components[7];
				}
				else {
					temp_record.mismatch = "x";
				}
				results.add(temp_record);
    		}
    		r.close();
    	}
    	catch (FileNotFoundException fnfe) {
    		System.err.println("Cannot find alignment file "+filename+".");
    		System.exit(1);
    	}
    	catch (IOException ioe) {
    		System.err.println("Alignment file "+filename+" contains an error.");
    		System.exit(1);
    	}
    	return results;
    }

    // Computes confidence based on match/mismatch; quality of read
    public static double[] compute_quality(ArrayList<String> q, ArrayList<String> m) {
		double[] result = new double[q.size()];
		for (int i=0; i<q.size(); i++) {
			result[i] = 1; // initializing for multiplication
			// check for mismatches
			int[] positions = {-1};
			if (!m.get(i).equals("x")) {
				// process mismatches
				String[] mismatch = m.get(i).split(",");
				positions = new int[mismatch.length];
				for (int j=0; j<mismatch.length; j++) {
					// get position of mismatch - part before : should be position
					String[] temp = mismatch[j].split(":");
					positions[j] = Integer.parseInt(temp[0]);
				}
			}
			// process quality scores
			int index = 0; // for positions[], assume listed in increasing order
			for (int j=0; j<q.get(i).length(); j++) {
				int quality = (int) q.get(i).charAt(j);
				quality -= 33; // because we have a +33 offset
				if (index >= positions.length || j != positions[index]) {
					// not a mismatch
					result[i] *= 1-Math.pow(10,(-quality/10.0));
				}
				else {
					// mismatch
					result[i] *= (1/3.0)*Math.pow(10,(-quality/10.0));
					index++;
				}
			}
		}
		return result;
    }

    public static Integer[] adjust_alignment(Integer[] r, int pos, int read_len,
    										SpliceGraph s, int fasta_readlen) {
		if (r.length == 1) {
			// st.get(i) is an index into that segment
			// do nothing, this is as simple as it gets
		}
		else {
			// find first segment used
			int j=0;
			Segment temp_1 = s.getSegment(r[0]);
			if (pos >= Math.min(temp_1.length(),fasta_readlen-1)) {
				pos -= Math.min(temp_1.length(),fasta_readlen-1);
				j++;
				temp_1 = s.getSegment(r[j]);
				while (pos >= temp_1.length() && temp_1.length() > 0) {
					if (temp_1.length() > 0) {
						pos -= Math.min(temp_1.length(),fasta_readlen-1);
						j++;
						temp_1 = s.getSegment(r[j]);
					}
					else {
						pos = 0;
					}
				}
			}
			
			// pos is now an index into the jth segment in r
			// (subsegment if j=0)
			int total = 0;
			if (j == 0) {
				total += Math.min(s.getSegment(r[0]).length(),fasta_readlen-1)-pos;
			}
			else {
				total += s.getSegment(r[j]).length()-pos;
			}
			int k=j;
			if (total >= read_len || s.getSegment(r[j]).length() == 0) {
				// then j is the only relevant segment
			}
			else {
				// let's find the end!
				while (total < read_len) {
					k++;
					temp_1 = s.getSegment(r[k]);
					if (temp_1.length() > 0) {
						total += temp_1.length();
					}
					else {
						// we've arrived at the poly-A tail!
						total = read_len;
					}
				}
			}
			
			// trim r between j and k
			Integer[] temp = new Integer[k-j+1];
			int ind = 0;
			for (int x=j; x<=k; x++) {
				temp[ind] = r[x];
				ind++;
			}
			r = temp;
		}
		return r;
	}
}